//
//
// scorm2004.js
//
// routines for runtime engine communication with a SCORM 2004 3rd Edition conformant LMS
//
//
//

// SCORM API adapter locals
var apiAdapter = null;		// a reference to the API adapter
var scormDebugMode = false; // true for additional messages


//
// lifecycle support functions
//
//
function initializeLMS() {
	doLMSInitialize();
}

function finalizeLMS() {
	doLMSSetValue("cmi.exit", "suspend");
	doLMSCommit();
	doLMSFinish();
}
	

// 
// RTE callback functions
//
//

function onLocationRequest()		{ return doLMSGetValue("cmi.location"); }
function onSuspendDataRequest()		{ return doLMSGetValue("cmi.suspend_data"); }
function onLocationChange(location)	{ doLMSSetValue("cmi.location", location); }
			
function onStateChange(stateData) {
// occurs when the lesson state has changed

	// always save state
	doLMSSetValue("cmi.suspend_data", stateData.suspendData);


	// set completion status
	//
	if (stateData.complete) {
	// lesson is complete
		doLMSSetValue("cmi.completion_status","completed");
		
	} else if ((stateData.scoreMax > 0) && (stateData.questionsComplete) && (stateData.passed)) {
	// mark lesson as complete if there are scored questions, all scored questions have been 
	// completed, and we have a passing grade -- even if the lesson as a whole isn't complete
		doLMSSetValue("cmi.completion_status","completed");
		
	} else {
	// lesson is not complete
		doLMSSetValue("cmi.completion_status","incomplete");
	}
	
	
	// set success status
	//
	if (stateData.scoreMax > 0) {
	// have scored questions
	
		if (stateData.passed) {
			doLMSSetValue("cmi.success_status","passed");
		} else {
			doLMSSetValue("cmi.success_status","failed");
			// doLMSSetValue("cmi.completion_status","incomplete"); // some users require incomplete if failed
		}
		doLMSSetValue("cmi.score.min",0);
		doLMSSetValue("cmi.score.max",stateData.scoreMax);
		doLMSSetValue("cmi.score.raw",stateData.scoreRaw);
		doLMSSetValue("cmi.score.scaled",(stateData.scoreRaw / stateData.scoreMax));
		
	} else {
	// no scored questions
		doLMSSetValue("cmi.success_status","passed");
	}

}









//
// api adapter wrapper functions
//
// each function attempts to perform the action, and either returns
// the result from the adapter, or false if there is no adapter
//
//


// initialize, terminate
// 
function doLMSInitialize() {
// the LMSInitialize function *must* be called before any others.  therefore,
// this function is the only one in this class that attempts to update the
// apiAdapter member variable - all others assume the adapter is valid

	if (apiAdapter == null) apiAdapter = getAPIAdapter();
	
	if (apiAdapter == null) {
		if (scormDebugMode) alert("Unable to locate API adapter.  LMSInitialize() not invoked.");
		return false;
	} else {
		return apiAdapter.Initialize("").toString();
	}
}

function doLMSFinish() {

	if (apiAdapter == null) {
		if (scormDebugMode) alert("API adapter not available.  LMSFinish() not invoked.");
		return false;
	} else {
		return apiAdapter.Terminate("").toString();
	}
}


// get value, set value
//
function doLMSGetValue(name) {

	if (apiAdapter == null) {
		if (scormDebugMode) alert("API adapter not available.\n  LMSGetValue(" + name + ") not invoked.");
		return "";
	} else {
		return apiAdapter.GetValue(name).toString();
	}
}

function doLMSSetValue(name,value) {

	if (apiAdapter == null) {
		if (scormDebugMode) alert("API adapter not available.\n  LMSSetValue(" + name + "," + value + ") not invoked.");
		return "";
	} else {
		return apiAdapter.SetValue(name,value).toString();
	}
}

function doLMSCommit() {

	if (apiAdapter == null) {
		if (scormDebugMode) alert("API adapter not available.  LMSCommit() not invoked.");
		return false;
	} else {
		return apiAdapter.Commit("").toString();
	}

}


// error handling
//
function doLMSGetLastErrorCode() {

	if (apiAdapter == null) {
		if (scormDebugMode) alert("API adapter not available.  LMSGetLastError() not invoked.");
		return false;
	} else {
		return apiAdapter.GetLastError().toString();
	}
}

function doLMSGetLastErrorString() {

	if (apiAdapter == null) {
		if (scormDebugMode) alert("API adapter not available.  LMSGetLastError() not invoked.");
		return false;
	} else {
		var errorCode = apiAdapter.GetLastError();
		return apiAdapter.GetErrorString(errorCode).toString();
	}
}


// api adapter locator functions
//
function getAPIAdapter() {
// searches for and returns the SCORM api adapter.  if not found within the current window
// hierarchy, searches within the window.opener hierarchy

	var adapter = findAPI(window);

	if ((adapter == null) && (window.opener != null) && (typeof(window.opener) != "undefined")) {
		adapter = findAPI(window.opener);
	}

	return adapter;
}

function findAPI(win) {
// recursively searches up the hierarchy for the api adapter.  limit of 500 tries 
// comes from the IEEE API standard
//
// returns the api adapter found, or null

	var findAPITries = 0;

  	while ((win.API_1484_11 == null) && (win.parent != null) && (win.parent != win)) {
		findAPITries++;
		if (findAPITries > 500) {
			if (scormDebugMode) alert("Error finding API adapter -- too deeply nested.");
			return null;
		}
		win = win.parent;
	}

	return win.API_1484_11;
}

